#[derive(::serde::Deserialize, Clone, PartialEq, ::prost::Message)]
pub struct KeyStringValuePair {
    #[prost(string, tag = "1")]
    pub key: ::prost::alloc::string::String,
    #[prost(string, tag = "2")]
    pub value: ::prost::alloc::string::String,
}
#[derive(::serde::Deserialize, Clone, PartialEq, ::prost::Message)]
pub struct Cpu {
    #[prost(double, tag = "2")]
    pub usage_percent: f64,
}
#[derive(::serde::Deserialize, Clone, PartialEq, ::prost::Message)]
pub struct Commands {
    #[prost(message, repeated, tag = "1")]
    pub commands: ::prost::alloc::vec::Vec<Command>,
}
#[derive(::serde::Deserialize, Clone, PartialEq, ::prost::Message)]
pub struct Command {
    #[prost(string, tag = "1")]
    pub command: ::prost::alloc::string::String,
    #[prost(message, repeated, tag = "2")]
    pub args: ::prost::alloc::vec::Vec<KeyStringValuePair>,
}
/// In most cases, detect point should be `server` or `client`.
/// Even in service mesh, this means `server`/`client` side sidecar
/// `proxy` is reserved only.
#[derive(
    ::serde::Deserialize,
    Clone,
    Copy,
    Debug,
    PartialEq,
    Eq,
    Hash,
    PartialOrd,
    Ord,
    ::prost::Enumeration,
)]
#[repr(i32)]
pub enum DetectPoint {
    Client = 0,
    Server = 1,
    Proxy = 2,
}
/// The segment is a collection of spans. It includes all collected spans in a simple one request context, such as a HTTP request process.
///
/// We recommend the agent/SDK report all tracked data of one request once for all, such as,
/// typically, such as in Java, one segment represent all tracked operations(spans) of one request context in the same thread.
/// At the same time, in some language there is not a clear concept like golang, it could represent all tracked operations of one request context.
#[derive(::serde::Deserialize, Clone, PartialEq, ::prost::Message)]
pub struct SegmentObject {
    /// A string id represents the whole trace.
    #[prost(string, tag = "1")]
    pub trace_id: ::prost::alloc::string::String,
    /// A unique id represents this segment. Other segments could use this id to reference as a child segment.
    #[prost(string, tag = "2")]
    pub trace_segment_id: ::prost::alloc::string::String,
    /// Span collections included in this segment.
    #[prost(message, repeated, tag = "3")]
    pub spans: ::prost::alloc::vec::Vec<SpanObject>,
    /// **Service**. Represents a set/group of workloads which provide the same behaviours for incoming requests.
    ///
    /// The logic name represents the service. This would show as a separate node in the topology.
    /// The metrics analyzed from the spans, would be aggregated for this entity as the service level.
    #[prost(string, tag = "4")]
    pub service: ::prost::alloc::string::String,
    /// **Service Instance**. Each individual workload in the Service group is known as an instance. Like `pods` in Kubernetes, it
    /// doesn't need to be a single OS process, however, if you are using instrument agents, an instance is actually a real OS process.
    ///
    /// The logic name represents the service instance. This would show as a separate node in the instance relationship.
    /// The metrics analyzed from the spans, would be aggregated for this entity as the service instance level.
    #[prost(string, tag = "5")]
    pub service_instance: ::prost::alloc::string::String,
    /// Whether the segment includes all tracked spans.
    /// In the production environment tracked, some tasks could include too many spans for one request context, such as a batch update for a cache, or an async job.
    /// The agent/SDK could optimize or ignore some tracked spans for better performance.
    /// In this case, the value should be flagged as TRUE.
    #[prost(bool, tag = "6")]
    pub is_size_limited: bool,
}
/// Segment reference represents the link between two existing segment.
#[derive(::serde::Deserialize, Clone, PartialEq, ::prost::Message)]
pub struct SegmentReference {
    /// Represent the reference type. It could be across thread or across process.
    /// Across process means there is a downstream RPC call for this.
    /// Typically, refType == CrossProcess means SpanObject#spanType = entry.
    #[prost(enumeration = "RefType", tag = "1")]
    pub ref_type: i32,
    /// A string id represents the whole trace.
    #[prost(string, tag = "2")]
    pub trace_id: ::prost::alloc::string::String,
    /// Another segment id as the parent.
    #[prost(string, tag = "3")]
    pub parent_trace_segment_id: ::prost::alloc::string::String,
    /// The span id in the parent trace segment.
    #[prost(int32, tag = "4")]
    pub parent_span_id: i32,
    /// The service logic name of the parent segment.
    /// If refType == CrossThread, this name is as same as the trace segment.
    #[prost(string, tag = "5")]
    pub parent_service: ::prost::alloc::string::String,
    /// The service logic name instance of the parent segment.
    /// If refType == CrossThread, this name is as same as the trace segment.
    #[prost(string, tag = "6")]
    pub parent_service_instance: ::prost::alloc::string::String,
    /// The endpoint name of the parent segment.
    /// **Endpoint**. A path in a service for incoming requests, such as an HTTP URI path or a gRPC service class + method signature.
    /// In a trace segment, the endpoint name is the name of first entry span.
    #[prost(string, tag = "7")]
    pub parent_endpoint: ::prost::alloc::string::String,
    /// The network address, including ip/hostname and port, which is used in the client side.
    /// Such as Client --> use 127.0.11.8:913 -> Server
    /// then, in the reference of entry span reported by Server, the value of this field is 127.0.11.8:913.
    /// This plays the important role in the SkyWalking STAM(Streaming Topology Analysis Method)
    /// For more details, read https://wu-sheng.github.io/STAM/
    #[prost(string, tag = "8")]
    pub network_address_used_at_peer: ::prost::alloc::string::String,
}
/// Span represents a execution unit in the system, with duration and many other attributes.
/// Span could be a method, a RPC, MQ message produce or consume.
/// In the practice, the span should be added when it is really necessary, to avoid payload overhead.
/// We recommend to creating spans in across process(client/server of RPC/MQ) and across thread cases only.
#[derive(::serde::Deserialize, Clone, PartialEq, ::prost::Message)]
pub struct SpanObject {
    /// The number id of the span. Should be unique in the whole segment.
    /// Starting at 0.
    #[prost(int32, tag = "1")]
    pub span_id: i32,
    /// The number id of the parent span in the whole segment.
    /// -1 represents no parent span.
    /// Also, be known as the root/first span of the segment.
    #[prost(int32, tag = "2")]
    pub parent_span_id: i32,
    /// Start timestamp in milliseconds of this span,
    /// measured between the current time and midnight, January 1, 1970 UTC.
    #[prost(int64, tag = "3")]
    pub start_time: i64,
    /// End timestamp in milliseconds of this span,
    /// measured between the current time and midnight, January 1, 1970 UTC.
    #[prost(int64, tag = "4")]
    pub end_time: i64,
    /// <Optional>
    /// In the across thread and across process, these references targeting the parent segments.
    /// The references usually have only one element, but in batch consumer case, such as in MQ or async batch process, it could be multiple.
    #[prost(message, repeated, tag = "5")]
    pub refs: ::prost::alloc::vec::Vec<SegmentReference>,
    /// A logic name represents this span.
    ///
    /// We don't recommend to include the parameter, such as HTTP request parameters, as a part of the operation, especially this is the name of the entry span.
    /// All statistic for the endpoints are aggregated base on this name. Those parameters should be added in the tags if necessary.
    /// If in some cases, it have to be a part of the operation name,
    /// users should use the Group Parameterized Endpoints capability at the backend to get the meaningful metrics.
    /// Read https://github.com/apache/skywalking/blob/master/docs/en/setup/backend/endpoint-grouping-rules.md
    #[prost(string, tag = "6")]
    pub operation_name: ::prost::alloc::string::String,
    /// Remote address of the peer in RPC/MQ case.
    /// This is required when spanType = Exit, as it is a part of the SkyWalking STAM(Streaming Topology Analysis Method).
    /// For more details, read https://wu-sheng.github.io/STAM/
    #[prost(string, tag = "7")]
    pub peer: ::prost::alloc::string::String,
    /// Span type represents the role in the RPC context.
    #[prost(enumeration = "SpanType", tag = "8")]
    pub span_type: i32,
    /// Span layer represent the component tech stack, related to the network tech.
    #[prost(enumeration = "SpanLayer", tag = "9")]
    pub span_layer: i32,
    /// Component id is a predefinited number id in the SkyWalking.
    /// It represents the framework, tech stack used by this tracked span, such as Spring.
    /// All IDs are defined in the https://github.com/apache/skywalking/blob/master/oap-server/server-bootstrap/src/main/resources/component-libraries.yml
    /// Send a pull request if you want to add languages, components or mapping defintions,
    /// all public components could be accepted.
    /// Follow this doc for more details, https://github.com/apache/skywalking/blob/master/docs/en/guides/Component-library-settings.md
    #[prost(int32, tag = "10")]
    pub component_id: i32,
    /// The status of the span. False means the tracked execution ends in the unexpected status.
    /// This affects the successful rate statistic in the backend.
    /// Exception or error code happened in the tracked process doesn't mean isError == true, the implementations of agent plugin and tracing SDK make the final decision.
    #[prost(bool, tag = "11")]
    pub is_error: bool,
    /// String key, String value pair.
    /// Tags provides more informance, includes parameters.
    ///
    /// In the OAP backend analysis, some special tag or tag combination could provide other advanced features.
    /// https://github.com/apache/skywalking/blob/master/docs/en/guides/Java-Plugin-Development-Guide.md#special-span-tags
    #[prost(message, repeated, tag = "12")]
    pub tags: ::prost::alloc::vec::Vec<KeyStringValuePair>,
    /// String key, String value pair with an accurate timestamp.
    /// Logging some events happening in the context of the span duration.
    #[prost(message, repeated, tag = "13")]
    pub logs: ::prost::alloc::vec::Vec<Log>,
    /// Force the backend don't do analysis, if the value is TRUE.
    /// The backend has its own configurations to follow or override this.
    ///
    /// Use this mostly because the agent/SDK could know more context of the service role.
    #[prost(bool, tag = "14")]
    pub skip_analysis: bool,
}
#[derive(::serde::Deserialize, Clone, PartialEq, ::prost::Message)]
pub struct Log {
    /// The timestamp in milliseconds of this event.,
    /// measured between the current time and midnight, January 1, 1970 UTC.
    #[prost(int64, tag = "1")]
    pub time: i64,
    /// String key, String value pair.
    #[prost(message, repeated, tag = "2")]
    pub data: ::prost::alloc::vec::Vec<KeyStringValuePair>,
}
/// A ID could be represented by multiple string sections.
#[derive(::serde::Deserialize, Clone, PartialEq, ::prost::Message)]
pub struct Id {
    #[prost(string, repeated, tag = "1")]
    pub id: ::prost::alloc::vec::Vec<::prost::alloc::string::String>,
}
/// The segment collections for trace report in batch and sync mode.
#[derive(::serde::Deserialize, Clone, PartialEq, ::prost::Message)]
pub struct SegmentCollection {
    #[prost(message, repeated, tag = "1")]
    pub segments: ::prost::alloc::vec::Vec<SegmentObject>,
}
/// Map to the type of span
#[derive(
    ::serde::Deserialize,
    Clone,
    Copy,
    Debug,
    PartialEq,
    Eq,
    Hash,
    PartialOrd,
    Ord,
    ::prost::Enumeration,
)]
#[repr(i32)]
pub enum SpanType {
    /// Server side of RPC. Consumer side of MQ.
    Entry = 0,
    /// Client side of RPC. Producer side of MQ.
    Exit = 1,
    /// A common local code execution.
    Local = 2,
}
/// Type of the reference
#[derive(
    ::serde::Deserialize,
    Clone,
    Copy,
    Debug,
    PartialEq,
    Eq,
    Hash,
    PartialOrd,
    Ord,
    ::prost::Enumeration,
)]
#[repr(i32)]
pub enum RefType {
    /// Map to the reference targeting the segment in another OS process.
    CrossProcess = 0,
    /// Map to the reference targeting the segment in the same process of the current one, just across thread.
    /// This is only used when the coding language has the thread concept.
    CrossThread = 1,
}
/// Map to the layer of span
#[derive(
    ::serde::Deserialize,
    Clone,
    Copy,
    Debug,
    PartialEq,
    Eq,
    Hash,
    PartialOrd,
    Ord,
    ::prost::Enumeration,
)]
#[repr(i32)]
pub enum SpanLayer {
    /// Unknown layer. Could be anything.
    Unknown = 0,
    /// A database layer, used in tracing the database client component.
    Database = 1,
    /// A RPC layer, used in both client and server sides of RPC component.
    RpcFramework = 2,
    /// HTTP is a more specific RPCFramework.
    Http = 3,
    /// A MQ layer, used in both producer and consuer sides of the MQ component.
    Mq = 4,
    /// A cache layer, used in tracing the cache client component.
    Cache = 5,
    /// A FAAS layer, used in function-as-a-Service platform.
    Faas = 6,
}
#[doc = r" Generated client implementations."]
pub mod trace_segment_report_service_client {
    #![allow(unused_variables, dead_code, missing_docs, clippy::let_unit_value)]
    use tonic::codegen::*;
    #[doc = " Define a trace segment report service."]
    #[doc = " All language agents or any trace collecting component, could use this service to send span collection to the SkyWalking OAP backend."]
    #[derive(Debug, Clone)]
    pub struct TraceSegmentReportServiceClient<T> {
        inner: tonic::client::Grpc<T>,
    }
    impl TraceSegmentReportServiceClient<tonic::transport::Channel> {
        #[doc = r" Attempt to create a new client by connecting to a given endpoint."]
        pub async fn connect<D>(dst: D) -> Result<Self, tonic::transport::Error>
        where
            D: std::convert::TryInto<tonic::transport::Endpoint>,
            D::Error: Into<StdError>,
        {
            let conn = tonic::transport::Endpoint::new(dst)?.connect().await?;
            Ok(Self::new(conn))
        }
    }
    impl<T> TraceSegmentReportServiceClient<T>
    where
        T: tonic::client::GrpcService<tonic::body::BoxBody>,
        T::ResponseBody: Body + Send + Sync + 'static,
        T::Error: Into<StdError>,
        <T::ResponseBody as Body>::Error: Into<StdError> + Send,
    {
        pub fn new(inner: T) -> Self {
            let inner = tonic::client::Grpc::new(inner);
            Self { inner }
        }
        pub fn with_interceptor<F>(
            inner: T,
            interceptor: F,
        ) -> TraceSegmentReportServiceClient<InterceptedService<T, F>>
        where
            F: tonic::service::Interceptor,
            T: tonic::codegen::Service<
                http::Request<tonic::body::BoxBody>,
                Response = http::Response<
                    <T as tonic::client::GrpcService<tonic::body::BoxBody>>::ResponseBody,
                >,
            >,
            <T as tonic::codegen::Service<http::Request<tonic::body::BoxBody>>>::Error:
                Into<StdError> + Send + Sync,
        {
            TraceSegmentReportServiceClient::new(InterceptedService::new(inner, interceptor))
        }
        #[doc = r" Compress requests with `gzip`."]
        #[doc = r""]
        #[doc = r" This requires the server to support it otherwise it might respond with an"]
        #[doc = r" error."]
        pub fn send_gzip(mut self) -> Self {
            self.inner = self.inner.send_gzip();
            self
        }
        #[doc = r" Enable decompressing responses with `gzip`."]
        pub fn accept_gzip(mut self) -> Self {
            self.inner = self.inner.accept_gzip();
            self
        }
        #[doc = " Recommended trace segment report channel."]
        #[doc = " gRPC streaming provides better performance."]
        #[doc = " All language agents should choose this."]
        pub async fn collect(
            &mut self,
            request: impl tonic::IntoStreamingRequest<Message = super::SegmentObject>,
        ) -> Result<tonic::Response<super::Commands>, tonic::Status> {
            self.inner.ready().await.map_err(|e| {
                tonic::Status::new(
                    tonic::Code::Unknown,
                    format!("Service was not ready: {}", e.into()),
                )
            })?;
            let codec = tonic::codec::ProstCodec::default();
            let path = http::uri::PathAndQuery::from_static(
                "/skywalking.v3.TraceSegmentReportService/collect",
            );
            self.inner
                .client_streaming(request.into_streaming_request(), path, codec)
                .await
        }
        #[doc = " An alternative for trace report by using gRPC unary"]
        #[doc = " This is provided for some 3rd-party integration, if and only if they prefer the unary mode somehow."]
        #[doc = " The performance of SkyWalking OAP server would be very similar with streaming report,"]
        #[doc = " the performance of the network and client side are affected"]
        pub async fn collect_in_sync(
            &mut self,
            request: impl tonic::IntoRequest<super::SegmentCollection>,
        ) -> Result<tonic::Response<super::Commands>, tonic::Status> {
            self.inner.ready().await.map_err(|e| {
                tonic::Status::new(
                    tonic::Code::Unknown,
                    format!("Service was not ready: {}", e.into()),
                )
            })?;
            let codec = tonic::codec::ProstCodec::default();
            let path = http::uri::PathAndQuery::from_static(
                "/skywalking.v3.TraceSegmentReportService/collectInSync",
            );
            self.inner.unary(request.into_request(), path, codec).await
        }
    }
}
#[derive(::serde::Deserialize, Clone, PartialEq, ::prost::Message)]
pub struct InstanceProperties {
    #[prost(string, tag = "1")]
    pub service: ::prost::alloc::string::String,
    #[prost(string, tag = "2")]
    pub service_instance: ::prost::alloc::string::String,
    #[prost(message, repeated, tag = "3")]
    pub properties: ::prost::alloc::vec::Vec<KeyStringValuePair>,
    /// Instance belong layer name which define in the backend, general is default.
    #[prost(string, tag = "4")]
    pub layer: ::prost::alloc::string::String,
}
#[derive(::serde::Deserialize, Clone, PartialEq, ::prost::Message)]
pub struct InstancePingPkg {
    #[prost(string, tag = "1")]
    pub service: ::prost::alloc::string::String,
    #[prost(string, tag = "2")]
    pub service_instance: ::prost::alloc::string::String,
    /// Instance belong layer name which define in the backend, general is default.
    #[prost(string, tag = "3")]
    pub layer: ::prost::alloc::string::String,
}
#[doc = r" Generated client implementations."]
pub mod management_service_client {
    #![allow(unused_variables, dead_code, missing_docs, clippy::let_unit_value)]
    use tonic::codegen::*;
    #[doc = " Define the service reporting the extra information of the instance."]
    #[derive(Debug, Clone)]
    pub struct ManagementServiceClient<T> {
        inner: tonic::client::Grpc<T>,
    }
    impl ManagementServiceClient<tonic::transport::Channel> {
        #[doc = r" Attempt to create a new client by connecting to a given endpoint."]
        pub async fn connect<D>(dst: D) -> Result<Self, tonic::transport::Error>
        where
            D: std::convert::TryInto<tonic::transport::Endpoint>,
            D::Error: Into<StdError>,
        {
            let conn = tonic::transport::Endpoint::new(dst)?.connect().await?;
            Ok(Self::new(conn))
        }
    }
    impl<T> ManagementServiceClient<T>
    where
        T: tonic::client::GrpcService<tonic::body::BoxBody>,
        T::ResponseBody: Body + Send + Sync + 'static,
        T::Error: Into<StdError>,
        <T::ResponseBody as Body>::Error: Into<StdError> + Send,
    {
        pub fn new(inner: T) -> Self {
            let inner = tonic::client::Grpc::new(inner);
            Self { inner }
        }
        pub fn with_interceptor<F>(
            inner: T,
            interceptor: F,
        ) -> ManagementServiceClient<InterceptedService<T, F>>
        where
            F: tonic::service::Interceptor,
            T: tonic::codegen::Service<
                http::Request<tonic::body::BoxBody>,
                Response = http::Response<
                    <T as tonic::client::GrpcService<tonic::body::BoxBody>>::ResponseBody,
                >,
            >,
            <T as tonic::codegen::Service<http::Request<tonic::body::BoxBody>>>::Error:
                Into<StdError> + Send + Sync,
        {
            ManagementServiceClient::new(InterceptedService::new(inner, interceptor))
        }
        #[doc = r" Compress requests with `gzip`."]
        #[doc = r""]
        #[doc = r" This requires the server to support it otherwise it might respond with an"]
        #[doc = r" error."]
        pub fn send_gzip(mut self) -> Self {
            self.inner = self.inner.send_gzip();
            self
        }
        #[doc = r" Enable decompressing responses with `gzip`."]
        pub fn accept_gzip(mut self) -> Self {
            self.inner = self.inner.accept_gzip();
            self
        }
        #[doc = " Report custom properties of a service instance."]
        pub async fn report_instance_properties(
            &mut self,
            request: impl tonic::IntoRequest<super::InstanceProperties>,
        ) -> Result<tonic::Response<super::Commands>, tonic::Status> {
            self.inner.ready().await.map_err(|e| {
                tonic::Status::new(
                    tonic::Code::Unknown,
                    format!("Service was not ready: {}", e.into()),
                )
            })?;
            let codec = tonic::codec::ProstCodec::default();
            let path = http::uri::PathAndQuery::from_static(
                "/skywalking.v3.ManagementService/reportInstanceProperties",
            );
            self.inner.unary(request.into_request(), path, codec).await
        }
        #[doc = " Keep the instance alive in the backend analysis."]
        #[doc = " Only recommend to do separate keepAlive report when no trace and metrics needs to be reported."]
        #[doc = " Otherwise, it is duplicated."]
        pub async fn keep_alive(
            &mut self,
            request: impl tonic::IntoRequest<super::InstancePingPkg>,
        ) -> Result<tonic::Response<super::Commands>, tonic::Status> {
            self.inner.ready().await.map_err(|e| {
                tonic::Status::new(
                    tonic::Code::Unknown,
                    format!("Service was not ready: {}", e.into()),
                )
            })?;
            let codec = tonic::codec::ProstCodec::default();
            let path =
                http::uri::PathAndQuery::from_static("/skywalking.v3.ManagementService/keepAlive");
            self.inner.unary(request.into_request(), path, codec).await
        }
    }
}
